@@ -166,11 +166,13 @@ class ImageFolder(Dataset):
                 tgt = self.target_mapping[target_name]
                 if self.target_transform:
                     tgt = self.target_transform(tgt)
+                # Keep as NumPy; Tensor will convert to xp internally
                 images.append(img)
                 targets.append(tgt)
 
-            X = Tensor(xp.stack(images, axis=0), dtype=self.img_dtype)
-            y = Tensor(xp.asarray(targets), dtype=self.target_dtype)
+            # Stack with NumPy; Tensor handles xp conversion
+            X = Tensor(np.stack(images, axis=0), dtype=self.img_dtype)
+            y = Tensor(np.asarray(targets), dtype=self.target_dtype)
             return X, y
 
         # Single index path
